/*
 * PluginSystem.cpp
 *
 *  Created on: 25 oct. 2010
 *      Author: guyet
 */

#include "PluginSystem.h"
#include <iostream>
#include <glibmm/fileutils.h>

PluginSystem::PluginSystem() {

}

PluginSystem::~PluginSystem() {
	//Destruction des objects chargés par le gestionnaire de plugin
	std::list<Class_Mere*>::iterator ip = objects.begin();
	while( ip!=objects.end() ) {
		delete(*ip);
		ip++;
	}
	objects.clear();

	//Destruction de la liste des modules
	std::list<Glib::Module*>::iterator ig = list_module.begin();
	while( ig != list_module.end() ) {
		delete(*ig); //Close the module
		ig++;
	}
}

bool PluginSystem::LoadModule(std::string path) {
	//Ouverture du module :
	Glib::Module *module = new Glib::Module(path, Glib::MODULE_BIND_MASK);
	if(! module) {
		std::cerr << "PluginSystem::LoadModule ouverture : " << module->get_last_error().c_str();
		std::cerr << std::endl;
		return false;
	}

	// On regarde si il y a une fonction GetObject() dans la table des symboles (de base,
	// c'est un pointeur, qui se révèle en fait être un functor
	void *functor=NULL;
	if( !module->get_symbol("GetObject", functor) ) {
		std::cerr << "PluginSystem::LoadModule pb chargement de GetObject : " << module->get_last_error().c_str();
		std::cerr << std::endl;
		delete(module);
		return false;
	}
	GetObject get_object_functor;

	// la fonction existe, on recupère l'outil
	get_object_functor=(GetObject)functor;
	Class_Mere* obj=get_object_functor();
	if( obj==NULL ) {
		std::cerr << "PluginSystem::LoadModule pb lors de l'utilisation du plugins : vérifier les versions (?)" << std::endl;
		delete(module);
		return false;
	}

	// on ne peut pas fermer le module tout de suite, donc on garde une référence pour le supprimer plus tard !
	list_module.push_back(module);

	// on ajoute le plugins à la liste des plugins
	objects.push_back(obj);
	return true;
}

bool PluginSystem::LoadPlugins(std::string path)
{
	std::list<std::string> list_file;
	if( LoadPath(path, &list_file) ) {
		std::list<std::string>::iterator f;
		for(f=list_file.begin(); f!=list_file.end(); ++f) {
			LoadModule( (gchar*)(*f).data() );
		}
		return true;
	}
	return false;
}

bool PluginSystem::LoadPath(std::string path, std::list<std::string> *list_file)
{
	//Si c'est un répertoire, alors on l'explore sinon retourne false
	if(!Glib::file_test(path,Glib::FILE_TEST_IS_DIR)) return false;

	Glib::Dir dir(path);
	std::list<std::string> entries (dir.begin(), dir.end());

	std::string path_and_dir(path);
	path_and_dir.append(G_DIR_SEPARATOR_S); //G_DIR_SEPARATOR_S s'adapte en fonction de windows ou Linux !
	std::string path_entier;
	std::list<std::string>::iterator it = entries.begin();
	while( it!=entries.end() ) {
		path_entier.clear();

		path_entier+=path_and_dir;
		path_entier.append( *it );

		//Appel récursif :
		if( !LoadPath(path_entier, list_file) ) {
			//ICI path_entier n'est pas un répertoire
			if( path_entier.find(".so")!=std::string::npos) // on ajoute le fichier à la liste si c'est un .so
				list_file->push_back( path_entier );
		}
		it++;
	}
	return true;
}

