/*
 * PluginSystem.h
 *
 *  Created on: 25 oct. 2010
 *      Author: guyet
 */

#ifndef PLUGINSYSTEM_H_
#define PLUGINSYSTEM_H_

#include <glibmm/module.h>
#include <list>
#include <string>
#include "Class_Mere.h"


//Profil de la fonction qu'il faut aller chercher
typedef Class_Mere* (*GetObject)();

class PluginSystem {
public:
	PluginSystem();

	/**
	 * \brief Destructeur
	 *
	 * Le desctructeur détruit les modules et les objets construits lors du chargement des modules.
	 */
	virtual ~PluginSystem();


	/**
	 * \brief Fonction de chargement des modules (fichiers de la forme .so)
	 * dans le répertoire path (et récursivement dans ses sous-répertoires)
	 * \param path chemin d'accès au répertoire des plugins (doit être relatif pour s'adapter à chaque installation)
	 * \return vrai si le chargement des plugins c'est bien passé et faux sinon
	 */
	bool LoadPlugins(std::string path);

	/**
	 * \brief Getter sur la liste des objets qui ont été chargés
	 */
	const std::list<Class_Mere*> getObjects() const {return objects;};

protected:
	/**
	 * Fonction d'exploration récursive des répertoires à la recherche des fichiers .so
	 * \param path chemin courant de recherche
	 * \param list_file pointeur sur la liste des fichiers qui ont été trouvés
	 * \return vrai si tout c'est bien passé et faux sinon
	 */
	bool LoadPath(std::string path, std::list<std::string> *list_file);

	/**
	 * Chargement unique d'un module identifié par son chemin
	 * \param path chemin d'un fichier .so à charger
	 * \return vrai si ca c'est bien passé, et faux sinon
	 *
	 * La fonction de chargement de module se passe bien si le fichier est
	 * effectivement un .so valide et si on y trouve une fonction "GetObject"
	 * dans la table des symboles.
	 */
	bool LoadModule(std::string path);

protected:
	std::list<Glib::Module*> list_module; 	//!< Liste des modules ouverts
	std::list<Class_Mere*> objects;			//!< liste des objets chargés
};

#endif /* PLUGINSYSTEM_H_ */
